@@ -6,7 +6,9 @@ population for genetic algorithm
 from __future__ import annotations
 import logging
 import numpy as np
-from typing import List
+import pandas as pd
+from typing import List, Union
+from copy import deepcopy
 from .individual import Individual
 
 logger = logging.getLogger('openGA')
@@ -14,14 +16,24 @@ logger = logging.getLogger('openGA')
 
 class Population(object):
     """
-    population contain current generation(includes adults, children) and next generateion
+    population contains individuals of current generation(includes adults, children) and next generation.
     """
 
     def __init__(self, gen_id: int, curr_gen: List[Individual], capacity: int = 20) -> Population:
+        """create population.
+
+        Args:
+            gen_id (int): generate id of current generation.
+            curr_gen (List[Individual]): list of individual of current generation.
+            capacity (int, optional): max individual could live in environment. Defaults to 20.
+
+        Returns:
+            Population: new population object.
+        """
         self._capacity = capacity
         self._gen_id = gen_id
         self._size = len(curr_gen)
-        self._curr_gen = curr_gen.copy()
+        self._curr_gen = deepcopy(curr_gen)
         self._parents = []
         self._children = []
         self._next_gen = []
@@ -29,6 +41,21 @@ class Population(object):
             self._curr_gen[i].gen_id = gen_id
             self._curr_gen[i].idv_id = i
 
+    def to_df(self, default_fit: float = np.nan) -> pd.DataFrame:
+        """convert population into dataframe with row of each individual of current generation.
+
+        Args:
+            default_fit (float, optional): default value for fitness when individual not grow-up. Defaults to np.nan.
+
+        Returns:
+            pd.DataFrame: population in dataframe format.
+        """
+        l = []
+        for p in self._curr_gen:
+            l.append(p.to_dict(default_fit=default_fit))
+        rlt = pd.DataFrame(l)
+        return rlt
+
     def __str__(self) -> str:
         rlt_str = f"population({self._size:d}/{self._capacity}):\n"
         rlt_str += f"current generation @{self._gen_id:d}:\n"
@@ -50,20 +77,46 @@ class Population(object):
 
     @property
     def gen_id(self) -> int:
+        """get generate id of `this` individual.
+
+        Returns:
+            int: generate id.
+        """
         return self._gen_id
 
     @property
     def curr_gen(self) -> List[Individual]:
+        """get shallow copy of individuals in current generation.
+
+        Returns:
+            List[Individual]: list of individuals.
+        """
         return self._curr_gen.copy()
 
     @property
     def parents(self) -> List[Individual]:
+        """get shallow copy of parent individuals.
+
+        Raises:
+            ValueError: error message of `parents not avialable, need select() in prior.`.
+
+        Returns:
+            List[Individual]: list of parent individuals.
+        """
         if not self._parents:
             raise ValueError('parents not avialable, need select() in prior.')
         return self._parents.copy()
 
     @property
     def children(self) -> List[Individual]:
+        """get shallow copy of children individuals.
+
+        Raises:
+            ValueError: error message of `children not available, need reproduce() in prior.`.
+
+        Returns:
+            List[Individual]: list of children individuals.
+        """
         if not self._children:
             raise ValueError(
                 'children not available, need reproduce() in prior.')
@@ -71,6 +124,14 @@ class Population(object):
 
     @property
     def next_gen(self) -> List[Individual]:
+        """get shallow copy of individuals in next generation.
+
+        Raises:
+            ValueError: error message of `next generation not available, need elimate() in prior.`.
+
+        Returns:
+            List[Individual]: list of individuals in next generation.
+        """
         if not self._next_gen:
             raise ValueError(
                 'next generation not available, need elimate() in prior.')
@@ -78,29 +139,61 @@ class Population(object):
 
     @staticmethod
     def evaluate(group: List[Individual]):
+        """Static Method. evaluate performance of individuals to update fitness.
+
+        Args:
+            group (List[Individual]): individuals for evalutation.
+        """
         for person in group:
             if not person.is_growup():
                 person.express()
                 person.evaluate()
 
     def size(self) -> int:
+        """get number of individuals in current generation.
+        """
         return self._size
 
     def append_newcomer(self, newcomer: Individual):
+        """add new individual into current generation as adult.
+
+        Args:
+            newcomer (Individual): new individual to be added.
+
+        Raises:
+            ValueError: error message of `invalid newcomer.`, if newcomer's plasm not match with that of individual in current generation.
+        """
         person = newcomer.copy()
         person.gen_id = self._gen_id
         person.idv_id = self._size
+        if self._curr_gen:
+            if not person.plasm.is_couple(self._curr_gen[0].plasm):
+                raise ValueError('invalid newcomer as reproduction isolation')
         self._curr_gen.append(person)
         self._size += 1
 
-    def append_newborn(self, newborn: Individual):
+    def _append_newborn(self, newborn: Individual):
+        """add new individual into current generation as child.
+
+        Args:
+            newborn (Individual): new individual to be added.
+        """
         person = newborn.copy()
         person.gen_id = self._gen_id
         person.idv_id = self._size
         self._children.append(person)
         self._size += 1
 
-    def select(self, pool_size: int = None, tour_size: int = 2) -> List[Individual]:
+    def select(self, pool_size: Union[int, None] = None, tour_size: int = 2) -> List[Individual]:
+        """select indvidual from current generation as parents for reproduce.
+
+        Args:
+            pool_size (Union[int, None], optional): number of parents. Defaults to None(1/2 of current generate individuals).
+            tour_size (int, optional): number of individual for tournament to be parents. Defaults to 2.
+
+        Returns:
+            List[Individual]: shallow copy of list of parent individuals.
+        """
         if pool_size is None:
             pool_size = int(self._size/2)
         # evaluate parents
@@ -129,6 +222,17 @@ class Population(object):
         return self._parents.copy()
 
     def reproduce(self, cross_prob: float = 0.9) -> List[Individual]:
+        """generate child individuals from parent by sexual reproduction.
+
+        Args:
+            cross_prob (float, optional): cross-over probability. Defaults to 0.9.
+
+        Raises:
+            ValueError: error message of `invalid cross_prob.`, when cross_prob not in [0, 1].
+
+        Returns:
+            List[Individual]: shallow copy of list of child indivudals.
+        """
         if not 0 <= cross_prob <= 1:
             raise ValueError(
                 'invalid cross_prob. Must in [0,1], got %6.4f' % cross_prob)
@@ -144,19 +248,24 @@ class Population(object):
                 # sexual produce
                 c0, c1 = self._parents[int(p0_idx)].sexual_reproduce(
                     self._parents[int(p1_idx)], p_mutation=0)
-                self.append_newborn(c0)
-                self.append_newborn(c1)
+                self._append_newborn(c0)
+                self._append_newborn(c1)
             else:
                 # asexual produce mutation
                 p_idx = np.random.randint(0, n_parents)
                 c = self._parents[int(p_idx)].asexual_reproduce(p_mutation=1)
                 c.idv_id = len(self._children) + self._size
-                self.append_newborn(c)
+                self._append_newborn(c)
         # evaluate children
         self.evaluate(self._children)
         return self._children.copy()
 
     def eliminate(self) -> List[Individual]:
+        """generate individuals in next generation by eliminating individuals with lower fitness in current generation and children group.
+
+        Returns:
+            List[Individual]: survivors in next generation with number of capacity.
+        """
         self._combine = self._curr_gen.copy()
         self._combine.extend(self._children)
         combine_fit_list = [idv.fitness for idv in self._combine]
@@ -167,13 +276,23 @@ class Population(object):
         # get the top size
         self._next_gen = []
         for i in range(self._capacity):
-            next_idv = self._combine[joint_sort[i][1]]
+            next_idv = deepcopy(self._combine[joint_sort[i][1]])
             next_idv.idv_id = i
             next_idv.gen_id = self._gen_id + 1
             self._next_gen.append(next_idv)
         return self._next_gen.copy()
 
-    def evolve(self, pool_size: int = None, tour_size: int = 2, cross_prob: float = 0.9) -> Population:
+    def evolve(self, pool_size: Union[int, None] = None, tour_size: int = 2, cross_prob: float = 0.9) -> Population:
+        """`this` population evolve into new population by select(), reproduce() and eliminate().
+
+        Args:
+            pool_size (Union[int, None], optional): number of parents. Defaults to None(1/2 of current generate individuals).
+            tour_size (int, optional): number of individual for tournament to be parents. Defaults to 2.
+            cross_prob (float, optional): cross-over probability. Defaults to 0.9.
+
+        Returns:
+            Population: new population after evolution.
+        """
         self.select(pool_size, tour_size)
         self.reproduce(cross_prob)
         survivors = self.eliminate()
