@@ -4,6 +4,7 @@ test case for genetic algorithm
 # Copyright (c) 2015-2021 Neal Nie. All rights reserved.
 
 import unittest
+from unittest.case import skip
 import numpy as np
 
 from openGA import GeneticAlgorithm
@@ -24,7 +25,17 @@ class TestGeneticAlgorithm(unittest.TestCase):
             'd': 0.3
         }
         self.ga = GeneticAlgorithm(base_gene)
+        self.raw_express = Individual.express
+        self.raw_evaluate = Individual.evaluate
 
+    def tearDown(self) -> None:
+        rec_df = self.ga.record
+        print(f'\nga record is:')
+        print(rec_df)
+        Individual.express = self.raw_express
+        Individual.evaluate = self.raw_evaluate
+
+    # @unittest.skip('debug')
     def test_init(self):
         self.assertFalse(self.ga._patched)
         self.assertFalse(self.ga.patched)
@@ -44,6 +55,22 @@ class TestGeneticAlgorithm(unittest.TestCase):
         print("evolution result:")
         print(f"{self.ga.result()}")
 
+    def test_append(self):
+        origin_size = self.ga.people.size()
+        base_gene = {
+            'a': 0.8,
+            'b': 0.9,
+            'c': 0.2,
+            'd': 0.7
+        }
+        self.ga.append(base_gene)
+        new_size = self.ga.people.size()
+        self.assertEqual(new_size, origin_size + 1)
+        self.assertDictEqual(
+            base_gene, self.ga.people.curr_gen[-1].plasm.to_dict())
+        self.assertIsNot(
+            base_gene, self.ga.people.curr_gen[-1].plasm.to_dict())
+
 
 if __name__ == "__main__":
     unittest.main()
