@@ -27,6 +27,12 @@ class TestIndividual(unittest.TestCase):
         plasm_1 = self.plasm_0.copy().mutate()
         self.idv_0 = Individual(plasm=self.plasm_0)
         self.idv_1 = Individual(plasm=plasm_1)
+        self.raw_express = Individual.express
+        self.raw_evaluate = Individual.evaluate
+
+    def tearDown(self) -> None:
+        Individual.express = self.raw_express
+        Individual.evaluate = self.raw_evaluate
 
     def test_init(self):
         self.assertEqual(self.idv_0.gen_id, -1)
@@ -40,6 +46,11 @@ class TestIndividual(unittest.TestCase):
         self.assertIs(self.idv_0._fitness, None)
         self.assertFalse(self.idv_1.is_growup())
 
+    def test_to_dict(self):
+        d = self.idv_0.to_dict()
+        self.assertTrue(d['fitness'] is np.nan)
+        print(f'\ndict is {d}')
+
     def test_plasm(self):
         plasm = self.idv_0.plasm
         self.assertListEqual(plasm._gene_names, self.gene_name)
@@ -72,6 +83,7 @@ class TestIndividual(unittest.TestCase):
         Individual.evaluate = evaluate
         self.idv_0.evaluate(4)
         self.assertEqual(self.idv_0.fitness, 4)
+        self.assertFalse(self.idv_0.to_dict()['fitness'] is np.nan)
 
     def test_str_format(self):
         print('\n%s' % self.idv_0)
